package ru.ifmo.fileShare.client;

import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;

import ru.ifmo.fileShare.Consts;
import ru.ifmo.fileShare.transport.Command;
import ru.ifmo.fileShare.transport.TransportUtil;

import com.evelopers.common.exception.CommonException;
import com.evelopers.unimod.core.stateworks.Event;
import com.evelopers.unimod.runtime.EventProvider;
import com.evelopers.unimod.runtime.ModelEngine;
import com.evelopers.unimod.runtime.context.StateMachineContext;
import com.evelopers.unimod.runtime.context.StateMachineContextImpl;

public class ClientSocket implements EventProvider {

	/**
     * @unimod.event.descr server said here
     */
	public static final String E131 = "e131";

	/**
     * @unimod.event.descr server sent list part
     */
	public static final String E132 = "e132";

    /**
     * @unimod.event.descr server sent last list part
     */
    public static final String E133 = "e133";

	/**
     * @unimod.event.descr server sent file part
     */
	public static final String E134 = "e134";

	/**
     * @unimod.event.descr server sent last file part
     */
	public static final String E135 = "e135";

	/**
     * @unimod.event.descr server said bye
     */
	public static final String E136 = "e136";

    private ModelEngine engine;
    private Socket socket;
    private String lastError;

    public void init(ModelEngine engine) throws CommonException {
    	this.engine = engine;
    	
		ClientController controller = (ClientController)engine.getControlledObjectsManager().getControlledObject("o1");
		controller.initSocket(this);
	}

	public void dispose() {
	}

	public boolean open(String hostname, int port) {
		try {
            lastError = null;
            socket = new Socket(InetAddress.getByName(hostname), port);
			startListener();
        } catch (UnknownHostException e) {
            lastError = "Unknown host: " + hostname;
            return false;
		} catch (IOException e) {
            lastError = "Network error occured: " + e.getMessage();
            return false;
        }
        return true;
	}

    public String getLastError() {
        return lastError;
    }

    public synchronized void closeConnection() {
        if (socket != null) {
            try {
                socket.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            socket = null;
        }
    }

	public void sayHello() {
		say(new Command(Command.CLIENT_HELLO));		
	}
	
	public void sayFirstListPart(long folderId) {
		say(new Command(Command.CLIENT_FIRST_LIST_PART, folderId));
	}
	
    public void sayNextListPart() {
        say(new Command(Command.CLIENT_NEXT_LIST_PART));
    }

	public void sayFirstFilePart(long fileId) {
		say(new Command(Command.CLIENT_FIRST_FILE_PART, fileId));
	}
	
	public void sayNextFilePart() {
		say(new Command(Command.CLIENT_NEXT_FILE_PART));
	}

    public void sayCancelDownload() {
        say(new Command(Command.CLIENT_CANCEL_FILE_DOWNLOAD));
    }

    public void sayBye() {
		say(new Command(Command.CLIENT_BYE));
	}
	
	private synchronized void say(Command command) {
		try {
			TransportUtil.sendCommand(socket.getOutputStream(), command);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private void startListener() {
		Thread thread = new Thread(new Runnable() {
			public void run() {
				try {
					while (socket != null) {
                        synchronized (ClientSocket.this) {
                            if (socket != null && socket.getInputStream().available() > 0) {
                                Command command = TransportUtil.receiveCommand(socket.getInputStream());
                                handleCommand(command);
                            }
                        }
                        Thread.sleep(50);
					}
				} catch (InterruptedException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		});
		thread.start();
	}
	
	private void handleCommand(Command command) {
		StateMachineContext context = StateMachineContextImpl.create();
		switch (command.getType()) {
			case Command.SERVER_HERE:
				engine.getEventManager().handle(new Event(E131), context);
				break;
			case Command.SERVER_LIST_PART:
				context.getEventContext().setParameter(Consts.LISTPART_PARAM, command.getData());
				engine.getEventManager().handle(new Event(E132), context);
				break;
            case Command.SERVER_LAST_LIST_PART:
                context.getEventContext().setParameter(Consts.LISTPART_PARAM, command.getData());
                engine.getEventManager().handle(new Event(E133), context);
                break;
			case Command.SERVER_FILE_PART:
				context.getEventContext().setParameter(Consts.FILEPART_PARAM, command.getData());
				engine.getEventManager().handle(new Event(E134), context);
				break;
			case Command.SERVER_LAST_FILE_PART:
				context.getEventContext().setParameter(Consts.FILEPART_PARAM, command.getData());
				engine.getEventManager().handle(new Event(E135), context);
				break;
			case Command.SERVER_BYE:
				engine.getEventManager().handle(new Event(E136), context);
				break;
		}
	}
}
