package ru.ifmo.fileShare.gui;

import ru.ifmo.fileShare.ClientEventListener;
import ru.ifmo.fileShare.client.ClientUserEventProvider;
import ru.ifmo.fileShare.gui.controls.*;
import ru.ifmo.fileShare.transport.FileListItem;
import ru.ifmo.fileShare.util.FolderInfo;

import javax.swing.*;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.table.TableCellRenderer;
import javax.swing.tree.ExpandVetoException;
import javax.swing.tree.TreePath;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ResourceBundle;

public class BrowserPane extends BaseResourcesPane implements ClientEventListener {
    private JButton connect;
    private JButton disconnect;
    private boolean connected;
    private WaitDialog waitDialog;
    private TreeNodeExpandState lastExpanded;
    private int expandingRow = -1;
    private int lastSelectedRow;
    private boolean clientQuited;
    private JButton copy;
    private JButton selectAll;
    private JButton unselectAll;
    private JButton refresh;

    private boolean refreshing;

    public BrowserPane() {
        props = ResourceBundle.getBundle("ru.ifmo.fileShare.gui.form");
        clientQuited = false;
    }

    protected JComponent createToolPane() {
        JPanel pane = new JPanel(new FlowLayout(FlowLayout.LEADING));
        connect = new JButton(props.getString("connect"));
        disconnect = new JButton(props.getString("disconnect"));
        copy = new JButton(props.getString("copy"));
        refresh = new JButton(props.getString("refresh"));
        pane.add(connect);
        pane.add(disconnect);
        pane.add(copy);
        pane.add(refresh);

        JPanel p = new JPanel(new BorderLayout());
        p.add(pane, BorderLayout.CENTER);
        JPanel selectionPane = new JPanel(new FlowLayout());
        selectAll = new JButton(props.getString("select-all"));
        unselectAll = new JButton(props.getString("unselect-all"));
        selectionPane.add(selectAll);
        selectionPane.add(unselectAll);
        p.add(selectionPane, BorderLayout.EAST);
        return p;
    }

    protected void setupControls() {
        super.setupControls();
        connect.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onConnectToServer();
            }
        });
        disconnect.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onDisconnectFromServer();
            }
        });
        copy.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onStartDownload();
            }
        });
        selectAll.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                selectFiles(true);
            }
        });
        unselectAll.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                selectFiles(false);
            }
        });
        refresh.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        refreshing = true;
                        updateRootFolders();
                    }
                });
            }
        });
    }

    private void selectFiles(boolean b) {
        fileTableModel.selectAll(b);
    }

    private String getCopyToFolder() {
        JFileChooser jf = new JFileChooser();
        jf.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        jf.setCurrentDirectory(getLastDir());
        if (JFileChooser.APPROVE_OPTION == jf.showOpenDialog(this)) {
            File f = jf.getSelectedFile();
            if (f != null) {
                setLastDir(jf.getSelectedFile());
                return f.getAbsolutePath();
            }
        }
        return null;
    }

    private void onStartDownload() {
        java.util.List<FolderInfo> selected = SharingApp.getInstance().getBrowserResourceController().getSelected();
        if (selected.isEmpty()) {
            JOptionPane.showMessageDialog(SharingApp.getInstance(), props.getString("nothing-selected"), props.getString("nothing-selected-title"), JOptionPane.INFORMATION_MESSAGE);
        } else {
            String folder = getCopyToFolder();
            if (folder != null) {
                SharingApp.getInstance().getBrowserResourceController().startDownload(selected, folder);
                updateData();
            }
        }
    }

    protected void updateControlState() {
        super.updateControlState();
        connect.setEnabled(!isConnected());
        disconnect.setEnabled(isConnected());
        refresh.setEnabled(isConnected());
        selectAll.setEnabled(fileTableModel.getRowCount() > 0);
        unselectAll.setEnabled(fileTableModel.getRowCount() > 0);
        copy.setEnabled(isConnected());
    }

    private boolean isConnected() {
        return connected;
    }

    private void onDisconnectFromServer() {
        doDisconnect();
    }

    private void onConnectToServer() {
        final JDialog jd = new ConnectDialog(new ConnectDialog.ConnectDlgListener() {
            public void onConnect(String hostName, String port) {
                doConnect(hostName, port);
            }
        });
        jd.setVisible(true);
    }

    private void doConnect(String hostName, String port) {
        ClientUserEventProvider.getInstance().connect(hostName, port);
        waitDialog = new WaitDialog(SharingApp.getInstance(), null);
        waitDialog.setTitle("Connecting to " + hostName + ":" + port);
        waitDialog.start();
    }

    private void doDisconnect() {
        ClientUserEventProvider.getInstance().disconnect();
    }

    public void onServerDisconnect() {
        JOptionPane.showMessageDialog(this, "Server closed session", "Session closed", JOptionPane.INFORMATION_MESSAGE);
        connected = false;
        updateControlState();
        onSessionClosed();
    }

    public void onClientQuited() {
        clientQuited = true;
    }

    public boolean hasClientQuited() {
        return clientQuited;
    }

    public void onServerDropped() {
        if (waitDialog != null) {
            waitDialog.close();
        }
        JOptionPane.showMessageDialog(this, "Server dropped due to timeout", "Timeout", JOptionPane.INFORMATION_MESSAGE);
        connected = false;
        updateControlState();
    }

    public void onConnectionError(String reason) {
        waitDialog.close();
        JOptionPane.showMessageDialog(this, reason, "Connection error", JOptionPane.INFORMATION_MESSAGE);
        connected = false;
        updateControlState();
    }

    public void onConnectedToServer() {
        waitDialog.close();
        connected = true;
        updateControlState();
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                updateRootFolders();
            }
        });
    }

    public void onSessionClosed() {
        SharingApp.getInstance().getBrowserResourceController().clearFolderInfo();
        updateData();
        connected = false;
        updateControlState();
    }

    public void onListRecieved(FileListItem[] fileListItems) {
        if (SharingApp.getInstance().getBrowserResourceController().fileListUpdate(fileListItems)) {
            updateData(refreshing);
            refreshing = false;
        }
        waitDialog.close();
    }

    public void onFileRecieved() {
        SharingApp.getInstance().getBrowserResourceController().onFileReceived();
    }

    public void onFilePartRecieved(long downloadedSize) {
        SharingApp.getInstance().getBrowserResourceController().onFilePartReceived(downloadedSize);
    }

    private void updateRootFolders() {
        waitDialog = new WaitDialog(SharingApp.getInstance(), null);
        SharingApp.getInstance().getBrowserResourceController().startLoadRootFolders();
        waitDialog.start();
    }

    private void updateSubFolder(FolderInfo fi) {
        waitDialog = new WaitDialog(SharingApp.getInstance(), null);
        SharingApp.getInstance().getBrowserResourceController().startLoadSubFolders(fi);
        waitDialog.start();
    }

    public void updateData() {
        updateData(false);
    }

    public void updateData(boolean reset) {
        lastExpanded = TreeNodeExpandState.getNodesExpandedState(folderTree);
        if (folderTree.getSelectionRows() != null) {
            lastSelectedRow = folderTree.getSelectionRows()[0];
        } else {
            lastSelectedRow = -1;
        }
        folderTreeModel.updateData(SharingApp.getInstance().getBrowserResourceController().getFolderInfo());
        if (!reset) {
            if (lastExpanded != null) {
                TreeNodeExpandState.setNodesExpandedState(folderTree, lastExpanded);
                if (expandingRow != -1)
                    folderTree.expandRow(expandingRow);
            }
            if (lastSelectedRow != -1) {
                (folderTree.getSelectionModel()).setSelectionPath(folderTree.getPathForRow(lastSelectedRow));
            }
        }
        fileTableModel.updateData();
    }

    protected void create() {
        super.create();
        folderTree.setCellRenderer(new FolderTreeCellRenderer());
        folderTree.setCellEditor(new FolderTreeCellEditor(folderTree));
        folderTree.setEditable(true);

        folderTree.addTreeWillExpandListener(new TreeWillExpandListener() {
            public void treeWillExpand(TreeExpansionEvent event) throws ExpandVetoException {
                FolderInfoTreeNode node = (FolderInfoTreeNode) event.getPath().getLastPathComponent();
                FolderInfo folderInfo = node.getFolderInfo();
                if (folderInfo != null && folderInfo.isDirectory()) {
                    if (folderInfo.hasChildren() && !folderInfo.isLoaded()) {
                        expandingRow = folderTree.getRowForPath(new TreePath(node.getPath()));
                        updateSubFolder(folderInfo);
                    }
                }
            }

            public void treeWillCollapse(TreeExpansionEvent event) throws ExpandVetoException {
            }
        });

        filesTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        filesTable.getTableHeader().setReorderingAllowed(false);
        filesTable.getColumnModel().getColumn(0).setPreferredWidth(25);
        filesTable.getColumnModel().getColumn(0).setResizable(false);
        filesTable.getColumnModel().getColumn(1).setPreferredWidth(150);
        filesTable.getColumnModel().getColumn(2).setPreferredWidth(150);
        filesTable.getColumnModel().getColumn(3).setPreferredWidth(150);

        filesTable.getColumnModel().getColumn(0).setCellEditor(new DefaultCellEditor(new JCheckBox()) {
            public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
                FolderInfo fi = (FolderInfo) value;
                return super.getTableCellEditorComponent(table, fi.isSelected(), isSelected, row, column);
            }
        });
        filesTable.getColumnModel().getColumn(0).setCellRenderer(new TableCellRenderer() {
            JCheckBox cbx = new JCheckBox();

            public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
                FolderInfo fi = (FolderInfo) value;
                cbx.setSelected(fi.isParentSelected());
                cbx.setEnabled(!fi.isDisabled());
                return cbx;
            }
        });

        folderTreeModel.addTreeModelListener(new TreeModelListener() {
            public void treeNodesChanged(TreeModelEvent e) {
                fileTableModel.updateData();
            }

            public void treeNodesInserted(TreeModelEvent e) {
            }

            public void treeNodesRemoved(TreeModelEvent e) {
            }

            public void treeStructureChanged(TreeModelEvent e) {
            }
        });
    }

    protected void onTreeSelectionChanged(Object userObject) {
        FolderInfo selectedFolder = (FolderInfo) userObject;
        if (selectedFolder != null) {
            if (selectedFolder.isDirectory()) {
                if (selectedFolder.hasChildren() && !selectedFolder.isLoaded()) {
                    expandingRow = -1;
                    updateSubFolder(selectedFolder);
                }
            }
        }

        fileTableModel.updateData(selectedFolder);
        updateFileListStatus();
        updateControlState();
    }

    protected boolean allowFileSelection() {
        return true;
    }
}
