package ru.ifmo.fileShare.gui;

import org.apache.commons.lang.BooleanUtils;
import org.apache.commons.lang.math.NumberUtils;
import org.apache.log4j.PropertyConfigurator;
import ru.ifmo.fileShare.ClientEventListener;
import ru.ifmo.fileShare.ClientModelEventProcessor;
import ru.ifmo.fileShare.client.BrowserResourceController;
import ru.ifmo.fileShare.client.ClientUserEventProvider;
import ru.ifmo.fileShare.server.ServerSocketAcceptor;
import ru.ifmo.fileShare.server.ServerUserEventProvider;
import ru.ifmo.fileShare.server.SharedResourceController;
import ru.ifmo.fileShare.util.WindowUtils;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;

public class SharingApp extends JFrame {
    private ResourceBundle props;
    private static final int WIDTH = 800;
    private static final int HEIGHT = 500;
    private Map<ClientInfoPane, ServerUserEventProvider> serverEventProviderMap;
    private BrowserPane browserPane;
    private JTabbedPane tabs;

    private static SharingApp instance;
    private SharedResourcesPane sharedResourcesFrame;
    private SharedResourceController sharedResourceController;
    private BrowserResourceController browserResourceController;

    public void dispose() {
        super.dispose();
        closeApplication();
    }

    private SharingApp() {
        super();

        ServerSocketAcceptor.startSocketAcceptor();

        serverEventProviderMap = new HashMap<ClientInfoPane, ServerUserEventProvider>();

        props = ResourceBundle.getBundle("ru.ifmo.fileShare.gui.form");
        setTitle(props.getString("title"));

        sharedResourceController = new SharedResourceController();
        browserResourceController = new BrowserResourceController();
    }

    public static void createInstance() {
        if (instance == null) {
            instance = new SharingApp();
        }
    }

    public static void destroyInstance() {
        if (instance != null) {
            getInstance().dispose();
        }
        instance = null;
    }

    public static SharingApp getInstance() {
        createInstance();
        return instance;
    }

    public void start() {
        create();
        setSize(new Dimension(WIDTH, HEIGHT));
        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        WindowUtils.locateAtScreenCenter(this);
        setVisible(true);
        PropertyConfigurator.configure("log4j.properties");
        performInitialTasks();
    }

    private void performInitialTasks() {
        try {
            //Run.main(new String[]{"A1.xml"});
            int debuggerPort = NumberUtils.stringToInt(System.getProperty("debugger.port"), -1);
            boolean debuggerSuspend = BooleanUtils.toBoolean(System.getProperty("debugger.suspend"));
            ClientModelEventProcessor.run(debuggerPort, debuggerSuspend);
        } catch (Exception e) {
            e.printStackTrace();
        }
        sharedResourceController.loadSharedFolderInfo();
        sharedResourceController.collectSharedFolderInfo(this, new Runnable() {
            public void run() {
                sharedResourceController.checkCollectedFodlerInfo();
                sharedResourcesFrame.updateData();
            }
        });
    }

    private void performTerminationTasks() {
        sharedResourceController.saveSharedFolderInfo();
    }

    private void create() {
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(createMainPane(), BorderLayout.CENTER);
        getContentPane().add(createToolPane(), BorderLayout.NORTH);
        getContentPane().add(createButtonPane(), BorderLayout.SOUTH);
    }

    private JComponent createButtonPane() {
        JPanel pane = new JPanel(new FlowLayout(FlowLayout.TRAILING));
        JButton closeButton = new JButton(props.getString("close"));
        closeButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dispose();
            }
        });
        pane.add(closeButton);
        return pane;
    }

    private void closeApplication() {
        performTerminationTasks();
        ServerSocketAcceptor.stopSocketAcceptor();
        ClientUserEventProvider.getInstance().quit();
        while (!browserPane.hasClientQuited()) {
            // waiting for client to shut down
            try {
                Thread.sleep(10);
            } catch (InterruptedException e) {
                // ignored
            }
        }
        try {
            Thread.sleep(50);
        } catch (InterruptedException e) {
            // ignored
        }
        for (ClientInfoPane cip : serverEventProviderMap.keySet()) {
            if (!cip.hasServerQuited()) {
                serverEventProviderMap.get(cip).disconnect();
            }
        }
        for (ClientInfoPane cip : serverEventProviderMap.keySet()) {
            while (!cip.hasServerQuited()) {
                // waiting for server to shut down
                try {
                    Thread.sleep(10);
                } catch (InterruptedException e) {
                    // ignored
                }
            }
        }
    }

    private JComponent createToolPane() {
        return new JPanel();
    }

    private JComponent createMainPane() {
        JPanel pane = new JPanel(new BorderLayout());
        tabs = new JTabbedPane();
        pane.add(tabs, BorderLayout.CENTER);

        tabs.add(props.getString("connect_server"), createClientPane());
        tabs.add(props.getString("shared_resources"), createSharedResourcePane());
        return pane;
    }

    private JComponent createClientPane() {
        browserPane = new BrowserPane();
        browserPane.create();
        return browserPane;
    }

    public ClientInfoPane createConnectedClientsPane() {
        return new ClientInfoPane();
    }

    private JComponent createSharedResourcePane() {
        sharedResourcesFrame = new SharedResourcesPane();
        sharedResourcesFrame.create();
        return sharedResourcesFrame;
    }

    public void addClientInfoPane(ClientInfoPane cip, ServerUserEventProvider suep) {
        tabs.add(props.getString("connected_client"), cip);
        serverEventProviderMap.put(cip, suep);
    }

    public void removeClientInfoPane(ClientInfoPane cip) {
        tabs.remove(cip);
    }

    public ServerUserEventProvider getServerUserEventProvider(ClientInfoPane cip) {
        return serverEventProviderMap.get(cip);
    }

    public ClientEventListener getClientEventListener() {
        return browserPane;
    }

    public SharedResourceController getSharedResourceController() {
        return sharedResourceController;
    }

    public BrowserResourceController getBrowserResourceController() {
        return browserResourceController;
    }

    public static void main(String[] args) {
        getInstance().start();
    }
}
