package ru.ifmo.fileShare.server;

import com.evelopers.unimod.runtime.ControlledObject;
import com.evelopers.unimod.runtime.context.StateMachineContext;
import ru.ifmo.fileShare.Consts;
import ru.ifmo.fileShare.ServerEventListener;
import ru.ifmo.fileShare.gui.SharingApp;
import ru.ifmo.fileShare.transport.TransportUtil;
import ru.ifmo.fileShare.util.ClientInfo;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Date;

public class ServerController implements ControlledObject {
    private ServerTimer timer;
    private ServerSocket socket;
    private FileInputStream fileStream;
    private byte[] list;
    private int offset;

    private ServerEventListener eventListener;

    public ServerController() {
        eventListener = ServerSocketAcceptor.getConnectingEventListener();
    }

    public boolean isInitialized() {
        return timer != null && socket != null;
    }

    /**
     * @param context state machine context
     * @unimod.action.descr say here to client
     */
    public void z1(StateMachineContext context) {
        eventListener.onClientConnected(
                new ClientInfo((String) context.getEventContext().getParameter(Consts.CLIENT_HOSTNAME_PARAM),
                        (String) context.getEventContext().getParameter(Consts.CLIENT_IP_PARAM),
                        (Date) context.getEventContext().getParameter(Consts.CLIENT_CONNECT_DATE_PARAM)));
        eventListener.onActionUpdate("Idle");
        socket.sayHere();
    }

    /**
     * @param context state machine context
     * @unimod.action.descr send list part to client
     */
    public void z2(StateMachineContext context) {
        eventListener.onActionUpdate("Getting list");
        socket.sendListPart(getListPart());
    }

    /**
     * @param context state machine context
     * @unimod.action.descr send last list part to client
     */
    public void z3(StateMachineContext context) {
        socket.sendLastListPart(getListPart());
        eventListener.onActionUpdate("Idle");
        list = null;
    }

    /**
     * @param context state machine context
     * @unimod.action.descr send file part to client
     */
    public void z4(StateMachineContext context) {
        eventListener.onActionUpdate("Getting file");
        socket.sendFilePart(getFilePart());
    }

    /**
     * @param context state machine context
     * @unimod.action.descr send last file part to client
     */
    public void z5(StateMachineContext context) {
        socket.sendLastFilePart(getFilePart());
        eventListener.onActionUpdate("Idle");
        closeStream();
    }

    /**
     * @param context state machine context
     * @unimod.action.descr cancel file sending
     */
    public void z6(StateMachineContext context) {
        eventListener.onActionUpdate("Idle");
        closeStream();
    }

    /**
     * @param context state machine context
     * @unimod.action.descr say bye to client
     */
    public void z7(StateMachineContext context) {
        socket.sayBye();
    }

    /**
     * @param context state machine context
     * @unimod.action.descr start timeout timer
     */
    public void z8(StateMachineContext context) {
        timer.startTimer(context);
    }

    /**
     * @param context state machine context
     * @unimod.action.descr timeout
     */
    public void z9(StateMachineContext context) {
        socket.closeConnection();
        closeStream();
        eventListener.onActionUpdate("Client session dropped due to timeout");
        eventListener.onSessionClosed();
    }

    /**
     * @param context state machine context
     * @unimod.action.descr remove timeout timer
     */
    public void z10(StateMachineContext context) {
        timer.clearTimer();
    }

    /**
     * @param context state machine context
     * @unimod.action.descr client quited
     */
    public void z11(StateMachineContext context) {
        socket.closeConnection();
        closeStream();
        eventListener.onActionUpdate("session closed by client");
        eventListener.onSessionClosed();
    }

    /**
     * @param context state machine context
     * @unimod.action.descr disconnect
     */
    public void z12(StateMachineContext context) {
        socket.closeConnection();
        closeStream();
        eventListener.onActionUpdate("session closed by server");
        eventListener.onSessionClosed();
    }

    /**
     * @param context state machine context
     * @return true if sending last file part, false otherwise
     * @unimod.action.descr sending last file part
     */
    public boolean x1(StateMachineContext context) {
        initFile(context);
        try {
            return fileStream != null && fileStream.available() <= Consts.FILE_PART_SIZE;
        } catch (IOException e) {
            e.printStackTrace();
            return false;
        }
    }

    /**
     * @param context state machine context
     * @return true if sending last list part, false otherwise
     * @unimod.action.descr sending last list part
     */
    public boolean x2(StateMachineContext context) {
        if (list == null) {
            initList(context);
        }
        return offset + Consts.LIST_PART_SIZE >= list.length;
    }

    public void initTimer(ServerTimer timer) {
        this.timer = timer;
    }

    public void initSocket(ServerSocket socket) {
        this.socket = socket;
    }

    private void initFile(StateMachineContext context) {
        if (fileStream == null) {
            try {
                Long fileId = (Long) context.getEventContext().getParameter(Consts.FILE_ID_PARAM);
                String filename = SharingApp.getInstance().getSharedResourceController().
                        getFolderInfoByID(fileId).getPath();
                fileStream = new FileInputStream(filename);
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }
        }
    }

    private byte[] getFilePart() {
        byte[] result = null;
        try {
            int size = Math.min(Consts.FILE_PART_SIZE, fileStream.available());
            result = new byte[size];
            fileStream.read(result, 0, size);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result;
    }

    private void initList(StateMachineContext context) {
        try {
            list = TransportUtil.toByteArray(SharingApp.getInstance().getSharedResourceController().getDirListByID(
                    (Long) context.getEventContext().getParameter(Consts.FOLDER_ID_PARAM)));
            offset = 0;
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private byte[] getListPart() {
        int length = Math.min(Consts.LIST_PART_SIZE, list.length - offset);
        byte[] result = new byte[length];
        System.arraycopy(list, offset, result, 0, length);
        offset += length;
        return result;
    }

    private void closeStream() {
        if (fileStream != null) {
            try {
                fileStream.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            fileStream = null;
        }
    }
}
