package ru.ifmo.fileShare.server;

import com.evelopers.common.exception.CommonException;
import com.evelopers.unimod.core.stateworks.Event;
import com.evelopers.unimod.runtime.EventProvider;
import com.evelopers.unimod.runtime.ModelEngine;
import com.evelopers.unimod.runtime.context.StateMachineContext;
import com.evelopers.unimod.runtime.context.StateMachineContextImpl;
import ru.ifmo.fileShare.Consts;
import ru.ifmo.fileShare.transport.Command;
import ru.ifmo.fileShare.transport.TransportUtil;

import java.io.IOException;
import java.net.Socket;
import java.util.Date;

public class ServerSocket implements EventProvider {

    /**
     * @unimod.event.descr client said hello
     */
    public static final String E231 = "e231";

    /**
     * @unimod.event.descr client requestes first list part
     */
    public static final String E232 = "e232";

    /**
     * @unimod.event.descr client requested next list part
     */
    public static final String E233 = "e233";

    /**
     * @unimod.event.descr client requested first file part
     */
    public static final String E234 = "e234";

    /**
     * @unimod.event.descr client requested next file part
     */
    public static final String E235 = "e235";

    /**
     * @unimod.event.descr client canceled file download
     */
    public static final String E236 = "e236";

    /**
     * @unimod.event.descr client said bye
     */
    public static final String E237 = "e237";

    private ModelEngine engine;
    private Socket socket;
    private ServerController controller;
    private boolean clientConnected;

    public void init(ModelEngine engine) throws CommonException {
        this.engine = engine;

        clientConnected = false;

        controller = (ServerController) engine.getControlledObjectsManager().getControlledObject("o2");
        controller.initSocket(this);

        socket = ServerSocketAcceptor.getConnectingSocket();
        clientConnected = true;
        startSocketListener();
    }

    public void dispose() {
    }

    public void closeConnection() {
        synchronized (this) {
            if (clientConnected) {
                clientConnected = false;
                try {
                    socket.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        // waiting for socket to close
        while (socket != null) {
            try {
                Thread.sleep(10);
            } catch (InterruptedException e) {
                // ignore
            }
        }
    }

    private void startSocketListener() {
        Thread thread = new Thread(new Runnable() {
            public void run() {
                try {
                    while (!controller.isInitialized()) {
                        try {
                            Thread.sleep(10);
                        } catch (Exception e) {
                            // ignore
                        }
                    }
                    while (clientConnected) {
                        synchronized (ServerSocket.this) {
                            if (socket.getInputStream().available() > 0) {
                                handleCommand(TransportUtil.receiveCommand(socket.getInputStream()));
                            }
                        }
                        try {
                            Thread.sleep(10);
                        } catch (Exception e) {
                            // ignore
                        }
                    }
                    socket = null;
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        });
        thread.start();
    }

    private void handleCommand(Command command) {
        StateMachineContext context = StateMachineContextImpl.create();
        switch (command.getType()) {
            case Command.CLIENT_HELLO:
                context.getEventContext().setParameter(Consts.CLIENT_HOSTNAME_PARAM, socket.getInetAddress().getHostName());
                context.getEventContext().setParameter(Consts.CLIENT_IP_PARAM, socket.getInetAddress().getHostAddress());
                context.getEventContext().setParameter(Consts.CLIENT_CONNECT_DATE_PARAM, new Date());
                context.getEventContext().setParameter(Consts.CLIENT_STATUS_PARAM, "Idle");
                engine.getEventManager().handle(new Event(E231), context);
                break;
            case Command.CLIENT_FIRST_LIST_PART:
                context.getEventContext().setParameter(Consts.FOLDER_ID_PARAM, command.getData());
                engine.getEventManager().handle(new Event(E232), context);
                break;
            case Command.CLIENT_NEXT_LIST_PART:
                engine.getEventManager().handle(new Event(E233), context);
                break;
            case Command.CLIENT_FIRST_FILE_PART:
                context.getEventContext().setParameter(Consts.FILE_ID_PARAM, command.getData());
                engine.getEventManager().handle(new Event(E234), context);
                break;
            case Command.CLIENT_NEXT_FILE_PART:
                engine.getEventManager().handle(new Event(E235), context);
                break;
            case Command.CLIENT_CANCEL_FILE_DOWNLOAD:
                engine.getEventManager().handle(new Event(E236), context);
                break;
            case Command.CLIENT_BYE:
                engine.getEventManager().handle(new Event(E237), context);
                break;
        }
    }

    public void sayHere() {
        say(new Command(Command.SERVER_HERE));
    }

    public void sendListPart(byte[] part) {
        say(new Command(Command.SERVER_LIST_PART, part));
    }

    public void sendLastListPart(byte[] part) {
        say(new Command(Command.SERVER_LAST_LIST_PART, part));
    }

    public void sendFilePart(byte[] part) {
        say(new Command(Command.SERVER_FILE_PART, part));
    }

    public void sendLastFilePart(byte[] part) {
        say(new Command(Command.SERVER_LAST_FILE_PART, part));
    }

    public void sayBye() {
        say(new Command(Command.SERVER_BYE));
    }

    private void say(Command command) {
        try {
            TransportUtil.sendCommand(socket.getOutputStream(), command);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
