package ru.ifmo.fileShare.util;

import ru.ifmo.fileShare.transport.FileListItem;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class FolderInfo {
    private static int fileIdSeq = 0;

    private String path;
    private String name;
    private boolean directory;

    private List<FolderInfo> children;
    private long length;
    private Date lastModified;

    private boolean selected;

    private FolderInfo parent;
    private long fileID;
    private boolean loaded;
    private boolean hasChildren;

    public FolderInfo(File f) {
        this(f, null);
    }

    public FolderInfo(File f, List<FolderInfo> children) {
        fileID = fileIdSeq++;
        this.path = f.getPath();
        this.name = f.getName();
        this.directory = f.isDirectory();
        setChildren(children);

        length = f.length();
        lastModified = new Date(f.lastModified());

        selected = false;

        loaded = true;
        hasChildren = false;
    }

    public FolderInfo(FileListItem item) {
        fileID = item.getFileID();
        path = item.getPath();
        name = item.getName();
        directory = item.isDirectory();
        length = item.getSize();
        lastModified = item.getModified();
        selected = false;
        loaded = !directory;
        hasChildren = item.hasChildren();
    }

    private void setParent(FolderInfo folderInfo) {
        parent = folderInfo;
    }

    public String getPath() {
        return path;
    }

    public String getName() {
        return name;
    }

    public boolean isDirectory() {
        return directory;
    }

    public List<FolderInfo> getChildren() {
        return children;
    }

    public long getLength() {
        return length;
    }

    public Date getLastModified() {
        return lastModified;
    }

    public String toString() {
        return getName();
    }

    public boolean isSelected() {
        return selected;
    }

    public void setSelected(boolean selected) {
        this.setSelected(selected, false);
    }

    public void setSelected(boolean selected, boolean fromParent) {
        this.selected = selected;
        selectChildren(false);
    }

    public boolean isDisabled() {
        return isParentSelected() && !isSelected();
    }

    public boolean isParentSelected() {
        return isSelected() || (parent != null && parent.isParentSelected());
    }

    public void toggleSelection() {
        setSelected(!isSelected());
    }

    private void selectChildren(boolean selected) {
        if (getChildren() != null) {
            for (FolderInfo fi : getChildren()) {
                fi.setSelected(selected, true);
            }
        }
    }

//    public void appendToList(List<FolderInfo> list) {
//        list.add(this);
//        if (getChildren() != null) {
//            list.addAll(getChildren());
//            for (FolderInfo fi: getChildren()) {
//                fi.appendToList(list);
//            }
//        }
//    }

    public FileListItem getListItem() {
        if (isDirectory()) {
            return new FileListItem(fileID, getPath(), getName(), getChildren() != null && !getChildren().isEmpty());
        } else {
            return new FileListItem(fileID, getPath(), getName(), getLength(), getLastModified());
        }
    }

    public FileListItem[] getDirList() {
        FileListItem[] list;
        if (getChildren() == null) {
            list = new FileListItem[0];
        } else {
            list = new FileListItem[getChildren().size()];
            List<FolderInfo> children = getChildren();
            int count = children.size();
            for (int i = 0; i < count; i++) {
                list[i] = children.get(i).getListItem();
            }
        }
        return list;
    }

    public FolderInfo getByPath(String path) {
        FolderInfo found = null;
        if (getPath().equals(path)) {
            found = this;
        } else {
            if (getChildren() != null) {
                for (FolderInfo fi : getChildren()) {
                    FolderInfo res = fi.getByPath(path);
                    if (res != null) {
                        found = res;
                        break;
                    }
                }
            }
        }
        return found;
    }

    public FolderInfo getByID(long id) {
        FolderInfo found = null;
        if (fileID == id) {
            found = this;
        } else {
            if (getChildren() != null) {
                for (FolderInfo fi : getChildren()) {
                    FolderInfo res = fi.getByID(id);
                    if (res != null) {
                        found = res;
                        break;
                    }
                }
            }
        }
        return found;
    }

    public static List<FolderInfo> wrap(FileListItem[] folderList) {
        List<FolderInfo> list = new ArrayList<FolderInfo>();
        for (FileListItem item : folderList) {
            list.add(new FolderInfo(item));
        }
        return list;
    }

    public void setChildren(FileListItem[] folderList) {
        loaded = true;
        children = wrap(folderList);
    }

    public boolean isLoaded() {
        return loaded;
    }

    public long getFileID() {
        return fileID;
    }

    public void setChildren(List<FolderInfo> children) {
        this.children = children;
        if (this.children != null) {
            for (FolderInfo fi : this.children) {
                fi.setParent(this);
            }
        }
        hasChildren = this.children != null && !this.children.isEmpty();
    }

    public void setLoaded(boolean loaded) {
        this.loaded = loaded;
    }

    public boolean hasChildren() {
        return hasChildren;
    }

    public FolderInfo getRoot() {
        if (parent == null) {
            return this;
        } else {
            return parent.getRoot();
        }
    }

    public String getRelativePath() {
        return getRelativePath(getRoot().getPath());
    }

    public String getRelativePath(String rootPath) {
        if (getPath().startsWith(rootPath)) {
            if (rootPath.equals(getPath())) {
                return getName();
            } else {
                return getPath().substring(rootPath.length());
            }
        } else {
            return getPath();
        }
    }

    public void getSelected(List<FolderInfo> selected) {
        if (isSelected()) {
            selected.add(this);
        } else {
            if (getChildren() != null) {
                for (FolderInfo fi : getChildren()) {
                    fi.getSelected(selected);
                }
            }
        }
    }

    public void getFiles(List<FolderInfo> files) {
        if (!isDirectory()) {
            files.add(this);
        } else {
            if (getChildren() != null) {
                for (FolderInfo fi : getChildren()) {
                    fi.getFiles(files);
                }
            }
        }
    }

    public boolean hasNotLoaded() {
        return false;
    }

    public FolderInfo getNotLoadedChild() {
        FolderInfo fi = null;
        if (!isLoaded()) {
            fi = this;
        } else {
            if (getChildren() != null) {
                for (FolderInfo child : getChildren()) {
                    FolderInfo childFi = child.getNotLoadedChild();
                    if (childFi != null) {
                        fi = childFi;
                    }
                }
            }
        }
        return fi;
    }

    public FolderInfo getParentSelected() {
        if (isSelected()) {
            return this;
        } else {
            if (parent != null) {
                return parent.getParentSelected();
            } else {
                return null;
            }
        }
    }
}
